var gPrinterControlInterface;
var gConsole;

window.addEventListener("load", function(e) {
    gConsole = new OnScreenConsole();
    gConsole.initialize('console');

    initCanvas();
    initUrl();

    IDAL.getControlInterfaces()
        .then(onControlInterfaceAvailable)
});

function onControlInterfaceAvailable(iface) {
    gPrinterControlInterface = iface.printer;

    window.addEventListener("idal-printer-event", onPrinterEvent);
    getAllJobStates();
    printerGetState();
}

function onPrinterEvent(e) {
    gConsole.log("Event 'idal-printer-event': " + JSON.stringify(e.detail));
    var detail = e.detail;
    switch (detail.type) {
        case 'job-state':
            //gConsole.log("Job " + detail.job.id + " is " + detail.job.state + (detail.job.message ? " (" + detail.job.message + ")" : ""));
            updateJobElement(detail.job.id, detail.job.state);
            break;

        case 'printer-state':
            updatePrinterStateElements(detail.state);
            //gConsole.log("Printer state: " + JSON.stringify(detail.state));
            break;

        default:
            gConsole.log("Unhandled event type=" + detail.type);
            break;
    }
}

function initUrl() {
    var images = [
        "image1.png",
        "image2.png",
        "not-existing-image.png"
    ];
    var urlPrefix = document.URL.substr(0, document.URL.lastIndexOf('/'));
    var select = document.getElementById("imageurlgroup");

    for (var i = 0 ; i < images.length ; i++) {
        var url = urlPrefix + "/" + images[i];
        var option = document.createElement("option");
        option.value = url;
        option.innerText = url;
        select.append(option);
    }
}

function genericHandleError(error) {
    gConsole.error("Error: ", JSON.stringify(error));
}

function printCanvas() {
    var canvas = document.getElementById('printcanvas');

    gConsole.log("Print canvas...");
    gPrinterControlInterface.print({data: canvas.toDataURL()})
        .then(function(id) {
            gConsole.log("Printer.print({data:...}) => " + id);
            addJobElement(id);
        })
        .catch(genericHandleError);
}

function getImageUrl() {
    var select = document.getElementById("imageurlselect");
    return url = select.options[select.selectedIndex].value;
}

function printUrl() {
    var url = getImageUrl();

    gConsole.log("Print URL " + url + "...");
    var param = { url: url };
    gPrinterControlInterface.print(param)
        .then(function(id) {
            gConsole.log("Printer.print(" + JSON.stringify(param) + ") => " + id);
            addJobElement(id);
        })
        .catch(genericHandleError);
}

function printerGetState() {
    //gConsole.log("Get printer state...");
    gPrinterControlInterface.getState()
        .then(function(state) {
            gConsole.log("Printer.getState() => " + JSON.stringify(state));
            updatePrinterStateElements(state)
        })
        .catch(genericHandleError);
}

function cancelAllJobs() {
    //gConsole.log("Cancel all jobs...");
    gPrinterControlInterface.cancelAllJobs()
        .then(function(value) {
            gConsole.log("Printer.cancelAllJobs() => " + value);
        })
        .catch(genericHandleError);
}

function cancelSelectedJob() {
    var printjobs = document.getElementById('printjobs');
    if ((printjobs.value !== undefined) && (printjobs.value != "")) {
        var id = parseInt(printjobs.value);
        //gConsole.log("Cancel job " + id + "...");
        gPrinterControlInterface.cancelJob(id)
            .then(function(result) {
                gConsole.log("Printer.cancelJob(" + id + ") => " + result);
                //removeJobElement(id);
            })
            .catch(genericHandleError);
    }
}

function getAllJobStates() {
    //gConsole.log("Get jobs...");

    removeAllJobElements();
    gPrinterControlInterface.getAllJobStates()
        .then(function(jobs) {
            gConsole.log("Printer.getAllJobStates() => " + JSON.stringify(jobs));
            gConsole.log(jobs.length + " jobs retrieved");
            for (var i = 0 ; i < jobs.length ; i++) {
                updateJobElement(jobs[i].id, jobs[i].state)
            }
        })
        .catch(genericHandleError);
}

function removeAllJobElements() {
    var jobgroup = document.getElementById('jobgroup');
    while (jobgroup.firstChild) {
        jobgroup.removeChild(jobgroup.firstChild);
    }
}

/*function removeJobElement(id) {
    var option = getJobElement(id);
    if (option !== undefined) {
        document.getElementById('jobgroup').removeChild(option);
    }
}*/

function addJobElement(id, state) {
    var jobgroup = document.getElementById('jobgroup');

    var option = document.createElement('option');
    option.value = id;
    option.innerText = getJobDescription(id, state);
    jobgroup.appendChild(option);
}

function getJobElement(id) {
    var options = document.getElementById('jobgroup').childNodes;

    for (var i = 0 ; i < options.length ; i++) {
        if (options[i].value == id) {
            return options[i];
        }
    }
}

function updateJobElement(id, state) {
    var current = getJobElement(id);
    if (current === undefined) {
        addJobElement(id, state);
    } else {
        current.innerText = getJobDescription(id, state);
    }
}

function getJobDescription(id, state) {
    return "Job " + id + " - " + state;
}

function loadUrlInCanvas() {
    var url = getImageUrl();
    gConsole.log("Load image in canvas: " + url);
    loadCanvas(url)
        .then(function() {
            gConsole.log("Image loaded successfully");
        })
        .catch(function() {
            gConsole.error("Could not load image!");
        });
}

function updatePrinterStateElements(state) {
    function setInnerText(id, value) {
        document.getElementById(id).innerText = value !== undefined ? value : "n/a";
    }
    setInnerText("printerconnected", state.connected);
    setInnerText("printeronline", state.online);
    setInnerText("printerpaper", state.paper);
    setInnerText("printercover", state.cover);
}
